# -*- coding: utf8 -*-

import re

from django.contrib import messages
from django.http import Http404
from django.shortcuts import render, redirect, get_object_or_404
from django.conf import settings
from django.urls import reverse
from django.utils.safestring import mark_safe

from lib.access_auth import permission_needed
from lib.tools import confirmation

from club.models import AliasGere
from ldap_inp import LdapAlias
from portailuser.app_logger import AppLogger
from ldap_inp import LdapEleve
from ldap_inp.ldapobject import L

from gestion.aliases.forms import NouvAliasForm, MailForm
from gestion.aliases.models import AssociationAdresseUser


# Logs
# _logger = AppLogger("Gestion des alias")
_logger = AppLogger("gestion_alias")


@permission_needed("gerer_aliases")
def list_aliases(request):
    aliases = sorted(LdapAlias.objects.filter(base=request.ecole.dn), key=lambda x: x.cn[0])

    if request.method == "POST":
        form = NouvAliasForm(request.ecole, request.POST)

        if form.is_valid():
            main_prefix = form.prefixes[0].decode("utf-8")
            LdapAlias(cn=form.prefixes, parent="ou=aliases," + request.ecole.dn).save()

            _logger.log(request.user, "A créé l'alias « {} »", main_prefix)
            messages.success(
                request,
                "L’alias « {} » a été créé avec succès."
                " Vous pouvez maintenant y ajouter des adresses.".format(main_prefix),
            )

            return redirect("gestion:aliases:edit", cn=main_prefix)

    else:
        form = NouvAliasForm(request.ecole)

    attente_confirmation = AliasGere.objects.filter(confirme=False)
    alias_associe = {}
    for associe in AliasGere.objects.filter(confirme=True):
        alias_associe[associe.prefixe] = [associe.club.nom] + (
            alias_associe[associe.prefixe] if alias_associe.get(associe.prefixe) else []
        )
    return render(request, "gestion/aliases/liste.html", locals())


@permission_needed("gerer_aliases")
def edit(request, cn):
    try:
        alias = LdapAlias.objects.get(cn=cn, base=request.ecole.dn)
    except LdapAlias.DoesNotExist:
        raise Http404

    alias.rfc822MailMember.sort()

    if request.method == "POST":
        form = MailForm(request.POST)
        if form.is_valid():
            if not form.cleaned_data["mail"]:
                user = form.cleaned_data["user"]
                mail = "{}@{}".format(user.username, settings.EMAIL_BDE[user.ecole.id - 1])
            else:
                mail = form.cleaned_data["mail"]

            if mail in alias.rfc822MailMember:
                messages.warning(request, "Cette adresse est déjà dans la liste.")
            else:
                alias.rfc822MailMember.append(mail)
                alias.save()
                search_login(alias, mail, False)  # on fait l'association de l'adresse avec la personne
                _logger.log(request.user, "A ajouté « {} » à l'alias {}", mail, cn)
                messages.success(
                    request,
                    "L’adresse « {} » a été ajoutée à " "l’alias avec succès.".format(mail),
                )

            return redirect("gestion:aliases:edit", cn=cn)
    else:
        form = MailForm()

    if request.GET.get("performance", None):
        list(liste_inscrits(alias, False))
        return redirect("gestion:aliases:edit", cn=cn)

    c = {
        "cn": cn,
        "members": liste_inscrits(alias),
        "form": form,
    }

    return render(request, "gestion/aliases/edit.html", c)


def search_login(alias, adresse, performance=True):
    # le performance permet de ne pas chercher les associations dans le ldap

    pseudos = AssociationAdresseUser.objects.filter(adresse=adresse).filter(alias=alias.cn[0])
    for pseudo in pseudos:
        try:
            return LdapEleve.objects.get(uid=pseudo.login)
        except LdapEleve.DoesNotExist:
            pseudo.delete()

    ident, domaine = adresse.rsplit("@", 1)
    if domaine in settings.EMAIL_BDE and re.match("^[a-z0-9]+$", ident):
        try:
            eleve = LdapEleve.objects.get(uid=ident)
            AssociationAdresseUser(alias=alias.cn[0], adresse=adresse, login=ident).save()
            return eleve
        except LdapEleve.DoesNotExist:
            pass

    if performance:
        return None

    eleves = LdapEleve.objects.filter(
        L(mailEcole=adresse) | L(mailLocal=adresse) | L(mailAnnexe=adresse) | L(mailForwardingAddress=adresse)
    )

    if len(eleves) == 1:
        AssociationAdresseUser(alias=alias.cn[0], adresse=adresse, login=eleves[0].uid).save()
        return eleves[0]
    return None


def liste_inscrits(alias, performance=True):
    """Liste des adresses / membres de l'alias ``alias``"""

    alias.rfc822MailMember.sort()

    for adresse in alias.rfc822MailMember:

        eleve = search_login(alias, adresse, performance)

        if eleve:
            infos = mark_safe(
                '<a href="{}">{} {} ({} {})</a>'.format(
                    reverse("annuaire:fiche", args=[eleve.uid]),
                    eleve.prenom,
                    eleve.nom,
                    eleve.filiere.get().shortName,
                    eleve.promo,
                )
            )

            yield infos, adresse
        else:
            yield "", adresse
            continue


@permission_needed("gerer_aliases")
def delete(request, cn):
    try:
        alias = LdapAlias.objects.get(cn=cn, base=request.ecole.dn)
    except LdapAlias.DoesNotExist:
        raise Http404

    message = "Êtes-vous sûr de vouloir supprimer l’alias « {} » ?".format(cn)
    boutons = [("conf_annul", "Annuler"), ("conf_ok", "Supprimer")]

    if request.method == "POST":
        if "conf_ok" in request.POST:
            alias.delete()
            AliasGere.objects.filter(prefixe=cn, club__ecole=request.ecole).delete()
            _logger.log(request.user, "A supprimé l'alias « {} »", cn)
            messages.success(request, "L’alias « {} » a été supprimé.".format(cn))

        return redirect("gestion:aliases:list_aliases")

    return confirmation(request, message, boutons, "Supprimer un alias")


@permission_needed("gerer_aliases")
def remove_addr(request, cn, mail):
    try:
        alias = LdapAlias.objects.get(cn=cn, base=request.ecole.dn)
    except LdapAlias.DoesNotExist:
        raise Http404

    if mail not in alias.rfc822MailMember:
        raise Http404

    if request.method == "POST":
        if "conf_ok" in request.POST:
            associations = AssociationAdresseUser.objects.filter(adresse=mail).filter(alias=alias.cn[0])
            for association in associations:
                association.delete()
            alias.rfc822MailMember.remove(mail)
            alias.save()
            _logger.log(request.user, "A supprimé « {} » de l'alias {}", mail, cn)
            messages.success(
                request,
                "L’adresse « {} » a été supprimée de " "l’alias avec succès.".format(mail),
            )

        return redirect("gestion:aliases:edit", cn=cn)

    message = "Êtes-vous sûr de vouloir supprimer « {} » de l’alias " "« {} » ?".format(mail, cn)
    boutons = [("conf_annul", "Annuler"), ("conf_ok", "Supprimer")]

    return confirmation(request, message, boutons, "Supprimer une adresse")


@permission_needed("gerer_aliases")
def confirm(request, pk):
    alias = get_object_or_404(AliasGere, pk=pk)
    alias.confirme = True
    alias.save()

    messages.success(
        request,
        "L'alias « {} » a été attribué au club « {} »".format(alias.prefixe, alias.club.nom),
    )
    return redirect("gestion:aliases:list_aliases")


@permission_needed("gerer_aliases")
def refuse(request, pk):
    alias = get_object_or_404(AliasGere, pk=pk)
    alias.delete()

    messages.success(
        request,
        "L'alias « {} » a été refusé au club « {} »".format(alias.prefixe, alias.club.nom),
    )
    return redirect("gestion:aliases:list_aliases")
